searchData={"items":[{"type":"behaviour","title":"Bamboo.Adapter","doc":"Behaviour for creating Bamboo adapters\n\nAll recipients in the `Bamboo.Email` struct will be normalized to a two item\ntuple of `{name, address}` when delivered through your mailer. For example,\n`elem(email.from, 0)` would return the name and `elem(email.from, 1)` would\nreturn the email address.\n\nFor more in-depth examples check out the\n[adapters in Bamboo](https://github.com/thoughtbot/bamboo/tree/master/lib/bamboo/adapters).","ref":"Bamboo.Adapter.html"},{"type":"behaviour","title":"Example - Bamboo.Adapter","doc":"defmodule Bamboo.CustomAdapter do\n      @behaviour Bamboo.Adapter\n\n      def deliver(email, config) do\n        deliver_the_email_somehow(email)\n      end\n\n      def handle_config(config) do\n        # Return the config if nothing special is required\n        config\n\n        # Or you could require certain config options\n        if Map.get(config, :smtp_username) do\n          config\n        else\n          raise \"smtp_username is required in config, got #{inspect(config)}\"\n        end\n      end\n\n      def supports_attachments?, do: true\n    end","ref":"Bamboo.Adapter.html#module-example"},{"type":"callback","title":"Bamboo.Adapter.deliver/2","doc":"","ref":"Bamboo.Adapter.html#c:deliver/2"},{"type":"callback","title":"Bamboo.Adapter.handle_config/1","doc":"","ref":"Bamboo.Adapter.html#c:handle_config/1"},{"type":"callback","title":"Bamboo.Adapter.supports_attachments?/0","doc":"","ref":"Bamboo.Adapter.html#c:supports_attachments?/0"},{"type":"type","title":"Bamboo.Adapter.error/0","doc":"","ref":"Bamboo.Adapter.html#t:error/0"},{"type":"module","title":"Bamboo.AdapterHelper","doc":"","ref":"Bamboo.AdapterHelper.html"},{"type":"function","title":"Bamboo.AdapterHelper.hackney_opts/1","doc":"","ref":"Bamboo.AdapterHelper.html#hackney_opts/1"},{"type":"module","title":"Bamboo.Attachment","doc":"","ref":"Bamboo.Attachment.html"},{"type":"function","title":"Bamboo.Attachment.new/2","doc":"Creates a new Attachment\n\n`content_id` can be used to embed an image, attach it and reference it in the message body by\nsetting its CID (Content-ID) and using a standard HTML tag:\n\n     \n\nwithin an HTML email message.\n\nExamples:\n\n    Bamboo.Attachment.new(\"/path/to/attachment.png\")\n    Bamboo.Attachment.new(\"/path/to/attachment.png\", filename: \"image.png\")\n    Bamboo.Attachment.new(\"/path/to/attachment.png\", filename: \"image.png\", content_type: \"image/png\", content_id: \"12387432\")\n    Bamboo.Attachment.new(params[\"file\"]) # Where params[\"file\"] is a %Plug.Upload\n\n    email\n    |> put_html_layout({LayoutView, \"email.html\"})\n    |> put_attachment(%Bamboo.Attachment{content_type: \"image/png\", filename: \"logo.png\", data: \"content\", content_id: \"2343333333\"})","ref":"Bamboo.Attachment.html#new/2"},{"type":"type","title":"Bamboo.Attachment.t/0","doc":"","ref":"Bamboo.Attachment.html#t:t/0"},{"type":"behaviour","title":"Bamboo.DeliverLaterStrategy","doc":"Behaviour for delivering emails with `Bamboo.Mailer.deliver_later/1`.\n\nUse this behaviour to create strategies for background email delivery. You\ncould make a strategy using a GenServer, a background job library or whatever\nelse you decide.","ref":"Bamboo.DeliverLaterStrategy.html"},{"type":"behaviour","title":"Bamboo ships with two strategies: - Bamboo.DeliverLaterStrategy","doc":"* `Bamboo.TaskSupervisorStrategy`\n* `Bamboo.ImmediateDeliveryStrategy`","ref":"Bamboo.DeliverLaterStrategy.html#module-bamboo-ships-with-two-strategies"},{"type":"behaviour","title":"Example of setting custom strategies - Bamboo.DeliverLaterStrategy","doc":"config :my_app, MyApp.Mailer,\n      adapter: Bamboo.MandrillAdapter, # or whatever adapter you want\n      deliver_later_strategy: MyCustomStrategy","ref":"Bamboo.DeliverLaterStrategy.html#module-example-of-setting-custom-strategies"},{"type":"behaviour","title":"Example of creating a custom strategy for delivering later using Task.async - Bamboo.DeliverLaterStrategy","doc":"defmodule Bamboo.TaskAsyncStrategy do\n      @behaviour Bamboo.DeliverLaterStrategy\n\n      # This is a strategy for delivering later using Task.async\n      def deliver_later(adapter, email, config) do\n        Task.async fn ->\n          # Always call deliver on the adapter so that the email is delivered.\n          adapter.deliver(email, config)\n        end\n      end\n    end","ref":"Bamboo.DeliverLaterStrategy.html#module-example-of-creating-a-custom-strategy-for-delivering-later-using-task-async"},{"type":"callback","title":"Bamboo.DeliverLaterStrategy.deliver_later/3","doc":"","ref":"Bamboo.DeliverLaterStrategy.html#c:deliver_later/3"},{"type":"module","title":"Bamboo.Email","doc":"Contains functions for composing emails.\n\nBamboo separates composing emails from delivering them. This separation makes\nemails easy to test and makes things like using a default layout or a default\nfrom address easy to do. This module is for creating emails. To actually send\nthem, use [Bamboo.Mailer](Bamboo.Mailer.html).","ref":"Bamboo.Email.html"},{"type":"module","title":"Handling email addresses - Bamboo.Email","doc":"The from, to, cc, and bcc addresses of a `Bamboo.Email` can be set to any\ndata structure for which there is an implementation of the\n[Bamboo.Formatter](Bamboo.Formatter.html) protocol or a list of such data\nstructures. Bamboo includes implementations for some common data structures\nor you can create your own. All from, to, cc, and bcc addresses are\nnormalized internally to a two item tuple of `{name, address}`. See\n[Bamboo.Formatter](Bamboo.Formatter.html) for more info.","ref":"Bamboo.Email.html#module-handling-email-addresses"},{"type":"module","title":"Simplest way to create a new email - Bamboo.Email","doc":"defmodule MyApp.Email do\n      import Bamboo.Email\n\n      def welcome_email(user) do\n        new_email(\n          from: \"me@app.com\",\n          to: user,\n          subject: \"Welcome!\",\n          text_body: \"Welcome to the app\",\n          html_body: \" Welcome to the app \"\n        )\n      end\n    end","ref":"Bamboo.Email.html#module-simplest-way-to-create-a-new-email"},{"type":"module","title":"Extracting common parts (default layout, default from address, etc.) - Bamboo.Email","doc":"Let's say you want all emails to have the same from address. Here's how you\ncould do that\n\n    defmodule MyApp.Email do\n      import Bamboo.Email\n\n      def welcome_email(user) do\n        # Since new_email/1 returns a struct you can update it with Kernel.struct!/2\n        struct!(base_email(),\n          to: user,\n          subject: \"Welcome!\",\n          text_body: \"Welcome to the app\",\n          html_body: \" Welcome to the app \"\n        )\n      end\n\n      def base_email do\n        new_email(from: \"me@app.com\")\n      end\n    end\n\nIn addition to keyword lists, `Bamboo.Email`s can also be built using function pipelines.\n\n    defmodule MyApp.Email do\n      import Bamboo.Email\n\n      def welcome_email(user) do\n        base_email()\n        |> to(user)\n        |> subject(\"Welcome!\")\n        |> text_body(\"Welcome to the app\")\n        |> html_body(\" Welcome to the app \")\n      end\n    end","ref":"Bamboo.Email.html#module-extracting-common-parts-default-layout-default-from-address-etc"},{"type":"function","title":"Bamboo.Email.all_recipients/1","doc":"Returns a list of all recipients (to, cc and bcc).","ref":"Bamboo.Email.html#all_recipients/1"},{"type":"function","title":"Bamboo.Email.bcc/2","doc":"Sets the `bcc` on the email.\n\n`bcc` receives as an argument any data structure for which\nthere is an implementation of the [`Bamboo.Formatter`](Bamboo.Formatter.html) protocol.\n\n    new_email()\n    |> bcc([\"sally@example.com\", \"james@example.com\"])","ref":"Bamboo.Email.html#bcc/2"},{"type":"function","title":"Bamboo.Email.block/1","doc":"Marks an email as blocked to prevent it from being sent out. This is meant to\nbe used from an interceptor. To learn more about interceptors, see the\n`Bamboo.Interceptor` behaviour.","ref":"Bamboo.Email.html#block/1"},{"type":"function","title":"Example - Bamboo.Email.block/1","doc":"iex> Bamboo.Email.block(%Bamboo.Email{blocked: false})\n    %Bamboo.Email{blocked: true}","ref":"Bamboo.Email.html#block/1-example"},{"type":"function","title":"Bamboo.Email.cc/2","doc":"Sets the `cc` on the email.\n\n`cc` receives as an argument any data structure for which\nthere is an implementation of the [`Bamboo.Formatter`](Bamboo.Formatter.html) protocol.\n\n    new_email()\n    |> cc([\"sally@example.com\", \"james@example.com\"])","ref":"Bamboo.Email.html#cc/2"},{"type":"function","title":"Bamboo.Email.from/2","doc":"Sets the `from` on the email.\n\n`from` receives as an argument any data structure for which\nthere is an implementation of the [`Bamboo.Formatter`](Bamboo.Formatter.html) protocol.\n\n    new_email()\n    |> from([\"sally@example.com\", \"james@example.com\"])","ref":"Bamboo.Email.html#from/2"},{"type":"function","title":"Bamboo.Email.get_address/1","doc":"Gets just the email address from a normalized email address\n\nNormalized email addresses are 2 item tuples {name, address}. This gets the\naddress part of the tuple. Use this instead of calling `elem(address, 1)`\nso that if Bamboo changes how email addresses are represented your code will\nstill work","ref":"Bamboo.Email.html#get_address/1"},{"type":"function","title":"Examples - Bamboo.Email.get_address/1","doc":"Bamboo.Email.get_address({\"Paul\", \"paul@thoughtbot.com\"}) # \"paul@thoughtbot.com\"","ref":"Bamboo.Email.html#get_address/1-examples"},{"type":"function","title":"Bamboo.Email.html_body/2","doc":"Sets the html_body on the email","ref":"Bamboo.Email.html#html_body/2"},{"type":"function","title":"Bamboo.Email.new_email/1","doc":"Used to create a new email\n\nIf called without arguments it is the same as creating an empty\n`%Bamboo.Email{}` struct. If called with arguments it will populate the struct\nwith given attributes.","ref":"Bamboo.Email.html#new_email/1"},{"type":"function","title":"Example - Bamboo.Email.new_email/1","doc":"# Same as %Bamboo.Email{from: \"support@myapp.com\"}\n    new_email(from: \"support@myapp.com\")","ref":"Bamboo.Email.html#new_email/1-example"},{"type":"function","title":"Bamboo.Email.put_attachment/2","doc":"Adds a data attachment to the email","ref":"Bamboo.Email.html#put_attachment/2"},{"type":"function","title":"Example - Bamboo.Email.put_attachment/2","doc":"put_attachment(email, %Bamboo.Attachment{})\n\nRequires the fields filename and data of the `%Bamboo.Attachment{}` struct to be set.","ref":"Bamboo.Email.html#put_attachment/2-example"},{"type":"function","title":"Example - Bamboo.Email.put_attachment/2","doc":"def create(conn, params) do\n      #...\n      email\n      |> put_attachment(%Bamboo.Attachment{filename: \"event.ics\", data: \"BEGIN:VCALENDAR...\"})\n      #...\n    end","ref":"Bamboo.Email.html#put_attachment/2-example"},{"type":"function","title":"Bamboo.Email.put_attachment/3","doc":"Adds a file attachment to the email","ref":"Bamboo.Email.html#put_attachment/3"},{"type":"function","title":"Example - Bamboo.Email.put_attachment/3","doc":"put_attachment(email, path, opts \\\\ [])\n\nAccepts `filename:  ` and `content_type:  ` options.\n\nIf you are using Plug, it accepts a Plug.Upload struct","ref":"Bamboo.Email.html#put_attachment/3-example"},{"type":"function","title":"Example - Bamboo.Email.put_attachment/3","doc":"def create(conn, params) do\n      #...\n      email\n      |> put_attachment(params[\"file\"])\n      #...\n    end","ref":"Bamboo.Email.html#put_attachment/3-example"},{"type":"function","title":"Bamboo.Email.put_header/3","doc":"Adds a header to the email","ref":"Bamboo.Email.html#put_header/3"},{"type":"function","title":"Example - Bamboo.Email.put_header/3","doc":"put_header(email, \"Reply-To\", \"support@myapp.com\")","ref":"Bamboo.Email.html#put_header/3-example"},{"type":"function","title":"Bamboo.Email.put_private/3","doc":"Adds a key/value to the private key of the email\n\nThis is mostly used to implement specific functionality for a particular\nadapter. It will rarely be used directly from your code. Internally this is\nused to set Mandrill specific params for the MandrillAdapter and it's also\nused to store the view module, template and layout when using Bamboo.Phoenix.","ref":"Bamboo.Email.html#put_private/3"},{"type":"function","title":"Example - Bamboo.Email.put_private/3","doc":"put_private(email, :tags, \"welcome-email\")","ref":"Bamboo.Email.html#put_private/3-example"},{"type":"function","title":"Bamboo.Email.subject/2","doc":"Sets the subject on the email","ref":"Bamboo.Email.html#subject/2"},{"type":"function","title":"Bamboo.Email.text_body/2","doc":"Sets the text_body on the email","ref":"Bamboo.Email.html#text_body/2"},{"type":"function","title":"Bamboo.Email.to/2","doc":"Sets the `to` on the email.\n\n`to` receives as an argument any data structure for which\nthere is an implementation of the [`Bamboo.Formatter`](Bamboo.Formatter.html) protocol.\n\n    new_email()\n    |> to([\"sally@example.com\", \"james@example.com\"])","ref":"Bamboo.Email.html#to/2"},{"type":"type","title":"Bamboo.Email.address/0","doc":"","ref":"Bamboo.Email.html#t:address/0"},{"type":"type","title":"Bamboo.Email.address_list/0","doc":"","ref":"Bamboo.Email.html#t:address_list/0"},{"type":"type","title":"Bamboo.Email.t/0","doc":"","ref":"Bamboo.Email.html#t:t/0"},{"type":"protocol","title":"Bamboo.Formatter","doc":"Converts data to email addresses.\n\nImplementations of the `Bamboo.Formatter` protocol convert a given data\nstructure to a two item tuple of `{name, address}` or an address string. The\n`opts` argument is a map with the key `:type` and a value of `:from`, `:to`,\n`:cc`, or `:bcc`. The options argument allows functions to pattern match an\naddress type and format a given data structure differently for different\ntypes of addresses.","ref":"Bamboo.Formatter.html"},{"type":"protocol","title":"Simple example - Bamboo.Formatter","doc":"Let's say you have a user struct like this.\n\n    defmodule MyApp.User do\n      defstruct first_name: nil, last_name: nil, email: nil\n    end\n\nBamboo can automatically format this struct if you implement the `Bamboo.Formatter`\nprotocol.\n\n    defimpl Bamboo.Formatter, for: MyApp.User do\n      # Used by `to`, `bcc`, `cc` and `from`\n      def format_email_address(user, _opts) do\n        fullname = \"#{user.first_name} #{user.last_name}\"\n        {fullname, user.email}\n      end\n    end\n\nNow you can create emails like this, and the user will be formatted correctly\n\n    user = %User{first_name: \"John\", last_name: \"Doe\", email: \"me@example.com\"}\n    Bamboo.Email.new_email(from: user)","ref":"Bamboo.Formatter.html#module-simple-example"},{"type":"protocol","title":"Customize formatting based on from, to, cc or bcc - Bamboo.Formatter","doc":"By pattern matching the `opts` argument, you can format a given data\nstructure differently for different types of addresses. For example, if you\nwant to provide the name of the app when sending email on behalf of a user,\nyou can format the name for all `type: :from` addresses.\n\n    defimpl Bamboo.Formatter, for: MyApp.User do\n      # Include the app name when used in a from address\n      def format_email_address(user, %{type: :from}) do\n        fullname = \"#{user.first_name} #{user.last_name}\"\n        {fullname <> \" (Sent from MyApp)\", user.email}\n      end\n\n      # Just use the name for all other types\n      def format_email_address(user, _opts) do\n        fullname = \"#{user.first_name} #{user.last_name}\"\n        {fullname, user.email}\n      end\n    end","ref":"Bamboo.Formatter.html#module-customize-formatting-based-on-from-to-cc-or-bcc"},{"type":"function","title":"Bamboo.Formatter.format_email_address/2","doc":"Receives data and opts and returns a string, a two item tuple `{name, address}`, or a list of either.\n\nopts is a map with the key `:type` and a value of\n`:from`, `:to`, `:cc` or `:bcc`. You can pattern match on this to customize\nthe address.","ref":"Bamboo.Formatter.html#format_email_address/2"},{"type":"type","title":"Bamboo.Formatter.opts/0","doc":"","ref":"Bamboo.Formatter.html#t:opts/0"},{"type":"type","title":"Bamboo.Formatter.t/0","doc":"All the types that implement this protocol.","ref":"Bamboo.Formatter.html#t:t/0"},{"type":"module","title":"Bamboo.ImmediateDeliveryStrategy","doc":"Strategy for `Bamboo.Mailer.deliver_later/1` that sends the email\nimmediately.\n\nThis strategy is used and required by the `Bamboo.LocalAdapter` and\n`Bamboo.TestAdapter`.","ref":"Bamboo.ImmediateDeliveryStrategy.html"},{"type":"behaviour","title":"Bamboo.Interceptor","doc":"Behaviour for creating an Interceptor.\n\nAn interceptor allows you to modify or block an email before it is sent. To\nblock an email, it must be marked as blocked with `Bamboo.Email.block/1`.","ref":"Bamboo.Interceptor.html"},{"type":"behaviour","title":"Example - Bamboo.Interceptor","doc":"defmodule Bamboo.DenyListInterceptor do\n      @behaviour Bamboo.Interceptor\n      @deny_list [\"bar@foo.com\"]\n\n      def call(email) do\n        if email.to in @deny_list do\n          Bamboo.Email.block(email)\n        else\n          email\n        end\n      end\n    end","ref":"Bamboo.Interceptor.html#module-example"},{"type":"callback","title":"Bamboo.Interceptor.call/1","doc":"","ref":"Bamboo.Interceptor.html#c:call/1"},{"type":"module","title":"Bamboo.LocalAdapter","doc":"Stores emails locally. Can be queried to see sent emails.\n\nUse this adapter for storing emails locally instead of sending them. Emails\nare stored and can be read from `Bamboo.SentEmail`. Typically this adapter is\nused in the dev environment so emails are not delivered to real email\naddresses.\n\nYou can use this adapter along with `Bamboo.SentEmailViewerPlug` to view\nemails in the browser.\n\nIf you want to open a new browser window for every new email, set the option\n`open_email_in_browser_url` to your preview path.","ref":"Bamboo.LocalAdapter.html"},{"type":"module","title":"Example config - Bamboo.LocalAdapter","doc":"# In config/config.exs, or config/dev.exs, etc.\n    config :my_app, MyApp.Mailer,\n      adapter: Bamboo.LocalAdapter,\n      open_email_in_browser_url: \"http://localhost:4000/sent_emails\" # optional\n\n    # Define a Mailer. Maybe in lib/my_app/mailer.ex\n    defmodule MyApp.Mailer do\n      use Bamboo.Mailer, otp_app: :my_app\n    end","ref":"Bamboo.LocalAdapter.html#module-example-config"},{"type":"function","title":"Bamboo.LocalAdapter.deliver/2","doc":"Adds email to `Bamboo.SentEmail`, can automatically open it in new browser tab","ref":"Bamboo.LocalAdapter.html#deliver/2"},{"type":"function","title":"Bamboo.LocalAdapter.handle_config/1","doc":"","ref":"Bamboo.LocalAdapter.html#handle_config/1"},{"type":"function","title":"Bamboo.LocalAdapter.supports_attachments?/0","doc":"","ref":"Bamboo.LocalAdapter.html#supports_attachments?/0"},{"type":"module","title":"Bamboo.Mailer","doc":"Functions for delivering emails using adapters and delivery strategies.\n\nAdds `deliver_now/1`, `deliver_now!/1`, `deliver_later/1` and\n`deliver_later!/1` functions to the mailer module in which it is used.\n\nBamboo [ships with several adapters][available-adapters]. It is also possible\nto create your own adapter.\n\nSee the [\"Getting Started\" section of the README][getting-started] for an\nexample of how to set up and configure a mailer for use.\n\n[available-adapters]: https://github.com/thoughtbot/bamboo/tree/master/lib/bamboo/adapters\n[getting-started]: https://hexdocs.pm/bamboo/readme.html#getting-started","ref":"Bamboo.Mailer.html"},{"type":"module","title":"Example - Bamboo.Mailer","doc":"Creating a Mailer is as simple as defining a module in your application and\nusing the `Bamboo.Mailer`.\n\n    # some/path/within/your/app/mailer.ex\n    defmodule MyApp.Mailer do\n      use Bamboo.Mailer, otp_app: :my_app\n    end\n\nThe mailer requires some configuration within your application.\n\n    # config/config.exs\n    config :my_app, MyApp.Mailer,\n      adapter: Bamboo.MandrillAdapter, # Specify your preferred adapter\n      api_key: \"my_api_key\" # Specify adapter-specific configuration\n\nAlso you will want to define an email module for building email structs that\nyour mailer can send. See [`Bamboo.Email`] for more information.\n\n    # some/path/within/your/app/email.ex\n    defmodule MyApp.Email do\n      import Bamboo.Email\n\n      def welcome_email do\n        new_email(\n          to: \"john@example.com\",\n          from: \"support@myapp.com\",\n          subject: \"Welcome to the app.\",\n          html_body: \" Thanks for joining! \",\n          text_body: \"Thanks for joining!\"\n        )\n      end\n    end\n\nYou are now able to send emails with your mailer module where you see fit\nwithin your application.","ref":"Bamboo.Mailer.html#module-example"},{"type":"function","title":"Bamboo.Mailer.build_config/3","doc":"","ref":"Bamboo.Mailer.html#build_config/3"},{"type":"function","title":"Bamboo.Mailer.deliver_later/2","doc":"Deliver an email in the background.\n\nCall your mailer with `deliver_later/1` to send an email using the configured\n`deliver_later_strategy`. If no `deliver_later_strategy` is set,\n`Bamboo.TaskSupervisorStrategy` will be used. See\n`Bamboo.DeliverLaterStrategy` to learn how to change how emails are delivered\nwith `deliver_later/1`.\n\nIf the email is successfully scheduled for delivery, this function will return\nan `{:ok, email}`.\n\nIf the email is invalid, this function will return an `{:error, error}` tuple.","ref":"Bamboo.Mailer.html#deliver_later/2"},{"type":"function","title":"Bamboo.Mailer.deliver_later!/2","doc":"Deliver an email in the background.\n\nSame as `deliver_later/2` but does not return an ok tuple and raises on\nerrors.\n\nIf successful, this function only returns the `Email` struct.\n\nIf the email is invalid, this function raises an error.","ref":"Bamboo.Mailer.html#deliver_later!/2"},{"type":"function","title":"Bamboo.Mailer.deliver_now/2","doc":"Deliver an email right away.\n\nCall your mailer with `deliver_now/1` to send an email right away. Call\n`deliver_later/1` if you want to send in the background.\n\nPass in an argument of `response: true` if you need access to the response\nfrom delivering the email.\n\nA successful email delivery returns an ok tuple with the `Email` struct and\nthe response (if `response: true`) from calling `deliver` with your adapter.\n\nA failure returns `{:error, error}` tuple.\n\nHaving the response returned from your adapter is useful if you need access to\nany data sent back from your email provider in the response.\n\n    {:ok, email, response} = Email.welcome_email |> Mailer.deliver_now(response: true)\n\nPass in an argument of `config: %{}` if you would like to dynamically override\nany keys in your application's default Mailer configuration.\n\n    Email.welcome_email\n    |> Mailer.deliver_now(config: %{username: \"Emma\", smtp_port: 2525})","ref":"Bamboo.Mailer.html#deliver_now/2"},{"type":"function","title":"Bamboo.Mailer.deliver_now!/2","doc":"Deliver an email right away.\n\nSame as `deliver_now/2` but does not return an ok/error tuple.\n\nIf successful, this function returns the `Email` struct or an `Email`,\nresponse tuple when setting `response: true`.\n\nOn failure, this function raises an error.","ref":"Bamboo.Mailer.html#deliver_now!/2"},{"type":"function","title":"Bamboo.Mailer.normalize_addresses/1","doc":"Wraps to, cc and bcc addresses in a list and normalizes email addresses.\n\nAlso formats the from address. Email normalization/formatting is done by\nimplementations of the [Bamboo.Formatter] protocol.","ref":"Bamboo.Mailer.html#normalize_addresses/1"},{"type":"module","title":"Bamboo.MailgunAdapter","doc":"Sends email using Mailgun's API.\n\nUse this adapter to send emails through Mailgun's API. Requires that an API\nkey and a domain are set in the config.\n\nSee `Bamboo.MailgunHelper` for extra functions that can be used by Bamboo.MailgunAdapter (tagging, merge vars, etc.)","ref":"Bamboo.MailgunAdapter.html"},{"type":"module","title":"Example config - Bamboo.MailgunAdapter","doc":"# In config/config.exs, or config.prod.exs, etc.\n    config :my_app, MyApp.Mailer,\n      adapter: Bamboo.MailgunAdapter,\n      api_key: \"my_api_key\" # or {:system, \"MAILGUN_API_KEY\"},\n      domain: \"your.domain\" # or {:system, \"MAILGUN_DOMAIN\"},\n      hackney_opts: [\n        recv_timeout: :timer.minutes(1)\n      ]\n\n    # Define a Mailer. Maybe in lib/my_app/mailer.ex\n    defmodule MyApp.Mailer do\n      use Bamboo.Mailer, otp_app: :my_app\n    end","ref":"Bamboo.MailgunAdapter.html#module-example-config"},{"type":"module","title":"API base URI configuration - Bamboo.MailgunAdapter","doc":"Mailgun makes a difference in the API base URL between sender\ndomains from within the EU and outside.\n\nBy default, the base URL is set to `https://api.mailgun.net/v3`.\nTo override this globally, you can use the Application environment:\n\n    Application.put_env(:bamboo, :mailgun_base_uri, \"https://api.eu.mailgun.net/v3\")\n\nHowever, for advanced configurations (for instance, for multi-tenant\nsetups where you pass in the adapter config when an email is sent),\nyou might want to specify this on the adapter level:\n\n    config :my_app, MyApp.Mailer,\n      adapter: Bamboo.MailgunAdapter,\n      api_key: \"my_api_key\",\n      domain: \"your.domain\",\n      base_uri: \"https://api.eu.mailgun.net/v3\"","ref":"Bamboo.MailgunAdapter.html#module-api-base-uri-configuration"},{"type":"function","title":"Bamboo.MailgunAdapter.deliver/2","doc":"","ref":"Bamboo.MailgunAdapter.html#deliver/2"},{"type":"function","title":"Bamboo.MailgunAdapter.filter_non_empty_mailgun_fields/1","doc":"","ref":"Bamboo.MailgunAdapter.html#filter_non_empty_mailgun_fields/1"},{"type":"module","title":"Bamboo.MailgunHelper","doc":"Functions for using features specific to Mailgun\n(e.g. tagging, templates).","ref":"Bamboo.MailgunHelper.html"},{"type":"function","title":"Bamboo.MailgunHelper.deliverytime/2","doc":"Schedule an email to be delivered in the future.\n\nMore details can be found in the\n[Mailgun documentation](https://documentation.mailgun.com/en/latest/user_manual.html#scheduling-delivery)","ref":"Bamboo.MailgunHelper.html#deliverytime/2"},{"type":"function","title":"Example - Bamboo.MailgunHelper.deliverytime/2","doc":"one_hour_from_now =\n      DateTime.utc_now()\n      |> DateTime.add(3600)\n\n    email\n    |> MailgunHelper.deliverytime(one_hour_from_now)","ref":"Bamboo.MailgunHelper.html#deliverytime/2-example"},{"type":"function","title":"Bamboo.MailgunHelper.recipient_variables/2","doc":"Adds Mailgun recipient variables to the outgoing email\n\nMore details can be found in the\n[Mailgun documentation](https://documentation.mailgun.com/en/latest/user_manual.html#recipient-variables)","ref":"Bamboo.MailgunHelper.html#recipient_variables/2"},{"type":"function","title":"Example - Bamboo.MailgunHelper.recipient_variables/2","doc":"variables = %{\n      \"user1@example.com\" => %{unique_id: \"ABC123456789\"},\n      \"user2@example.com\" => %{unique_id: \"ZXY987654321\"}\n    }\n\n    email\n    |> MailgunHelper.recipient_variables(variables)","ref":"Bamboo.MailgunHelper.html#recipient_variables/2-example"},{"type":"function","title":"Bamboo.MailgunHelper.substitute_variables/2","doc":"This behaves like `Bamboo.MailgunHelper.substitute_variables/3`, but\naccepts a `Map` rather than a key, value pair.","ref":"Bamboo.MailgunHelper.html#substitute_variables/2"},{"type":"function","title":"Example - Bamboo.MailgunHelper.substitute_variables/2","doc":"email\n    |> MailgunHelper.template(\"password-reset-email\")\n    |> MailgunHelper.substitute_variables(%{ \"greeting\" => \"Hello!\", \"password_reset_link\" => \"https://example.com/123\" })","ref":"Bamboo.MailgunHelper.html#substitute_variables/2-example"},{"type":"function","title":"Bamboo.MailgunHelper.substitute_variables/3","doc":"When sending an email with `Bamboo.MailgunHelper.template/2` you can\nreplace a handlebars variables using this function.\n\nMore details about templates can be found in the\n[Templates section of the Mailgun documentation](https://documentation.mailgun.com/en/latest/user_manual.html#templates).","ref":"Bamboo.MailgunHelper.html#substitute_variables/3"},{"type":"function","title":"Example - Bamboo.MailgunHelper.substitute_variables/3","doc":"email\n    |> MailgunHelper.template(\"password-reset-email\")\n    |> MailgunHelper.substitute_variables(\"password_reset_link\", \"https://example.com/123\")","ref":"Bamboo.MailgunHelper.html#substitute_variables/3-example"},{"type":"function","title":"Bamboo.MailgunHelper.tag/2","doc":"Add a tag to outgoing email to help categorize traffic based on some\ncriteria, perhaps separate signup emails from password recovery emails\nor from user comments.\n\nMore details can be found in the\n[Mailgun documentation](https://documentation.mailgun.com/en/latest/user_manual.html#tagging)","ref":"Bamboo.MailgunHelper.html#tag/2"},{"type":"function","title":"Example - Bamboo.MailgunHelper.tag/2","doc":"email\n    |> MailgunHelper.tag(\"welcome-email\")","ref":"Bamboo.MailgunHelper.html#tag/2-example"},{"type":"function","title":"Bamboo.MailgunHelper.template/2","doc":"Send an email using a template stored in Mailgun, rather than supplying\na `Bamboo.Email.text_body/2` or a `Bamboo.Email.html_body/2`.\n\nMore details about templates can be found in the\n[Templates section of the Mailgun documentation](https://documentation.mailgun.com/en/latest/user_manual.html#templates).","ref":"Bamboo.MailgunHelper.html#template/2"},{"type":"function","title":"Bamboo.MailgunHelper.template_text/2","doc":"Use it if you want to have rendered template in the text part of the\nmessage in case of template sending.\n\nMore details can be found in the\n[Mailgun documentation](https://documentation.mailgun.com/en/latest/api-sending.html#sending)","ref":"Bamboo.MailgunHelper.html#template_text/2"},{"type":"function","title":"Example - Bamboo.MailgunHelper.template_text/2","doc":"email\n    |> MailgunHelper.template_text(true)","ref":"Bamboo.MailgunHelper.html#template_text/2-example"},{"type":"function","title":"Bamboo.MailgunHelper.template_version/2","doc":"Use it to send a message to specific version of a template.\n\nMore details can be found in the\n[Mailgun documentation](https://documentation.mailgun.com/en/latest/api-sending.html#sending)","ref":"Bamboo.MailgunHelper.html#template_version/2"},{"type":"function","title":"Example - Bamboo.MailgunHelper.template_version/2","doc":"email\n    |> MailgunHelper.template(\"my-template\")\n    |> MailgunHelper.template_version(\"v2\")","ref":"Bamboo.MailgunHelper.html#template_version/2-example"},{"type":"module","title":"Bamboo.MandrillAdapter","doc":"Sends email using Mandrill's JSON API.\n\nUse this adapter to send emails through Mandrill's API. Requires that an API\nkey is set in the config. See `Bamboo.MandrillHelper` for extra functions that\ncan be used by `Bamboo.MandrillAdapter` (tagging, merge vars, etc.)","ref":"Bamboo.MandrillAdapter.html"},{"type":"module","title":"Example config - Bamboo.MandrillAdapter","doc":"# In config/config.exs, or config/prod.exs, etc.\n    config :my_app, MyApp.Mailer,\n      adapter: Bamboo.MandrillAdapter,\n      api_key: \"my_api_key\",\n      hackney_opts: [\n        recv_timeout: :timer.minutes(1)\n      ]\n\n    # Define a Mailer. Maybe in lib/my_app/mailer.ex\n    defmodule MyApp.Mailer do\n      use Bamboo.Mailer, otp_app: :my_app\n    end","ref":"Bamboo.MandrillAdapter.html#module-example-config"},{"type":"function","title":"Bamboo.MandrillAdapter.deliver/2","doc":"","ref":"Bamboo.MandrillAdapter.html#deliver/2"},{"type":"module","title":"Bamboo.MandrillHelper","doc":"Functions for using features specific to Mandrill (e.g. tagging, merge vars,\ntemplates).","ref":"Bamboo.MandrillHelper.html"},{"type":"function","title":"Bamboo.MandrillHelper.put_merge_vars/3","doc":"Set merge_vars that are used by Mandrill.","ref":"Bamboo.MandrillHelper.html#put_merge_vars/3"},{"type":"function","title":"Example - Bamboo.MandrillHelper.put_merge_vars/3","doc":"email\n    |> put_merge_vars(users, fn(user) -> %{first_name: user.first_name} end)\n\nA convenience function for:\n\n    email\n    |> put_param(\"merge_vars\", [\n      %{\n        rcpt: \"user1@example.com\",\n        vars: [\n          %{\n            \"name\": \"full_name\",\n            \"content\": \"User 1\"\n          }\n        ]\n      },\n      %{\n        rcpt: \"user2@example.com\",\n        vars: [\n          %{\n            \"name\": \"full_name\",\n            \"content\": \"User 2\"\n          }\n        ]\n      }\n    ])","ref":"Bamboo.MandrillHelper.html#put_merge_vars/3-example"},{"type":"function","title":"Bamboo.MandrillHelper.put_param/3","doc":"Put extra message parameters that are used by Mandrill.\n\nParameters set with this function are sent to Mandrill when used along with\nthe `Bamboo.MandrillAdapter`. You can set things like `important`,\n`merge_vars`, and whatever else you need that the Mandrill API supports.","ref":"Bamboo.MandrillHelper.html#put_param/3"},{"type":"function","title":"Example - Bamboo.MandrillHelper.put_param/3","doc":"email\n    |> put_param(\"track_opens\", true)\n    |> put_param(\"merge_vars\", [\n      %{\n        rcpt: \"recipient.email@example.com\",\n        vars: [\n          %{\n            \"name\": \"first_name\",\n            \"content\": \"John Doe\"\n          }\n        ]\n      }\n    ])","ref":"Bamboo.MandrillHelper.html#put_param/3-example"},{"type":"function","title":"Bamboo.MandrillHelper.tag/2","doc":"Set a single tag or multiple tags for an email.\n\nA convenience function for `put_param(email, \"tags\", [\"my-tag\"])`","ref":"Bamboo.MandrillHelper.html#tag/2"},{"type":"function","title":"Example - Bamboo.MandrillHelper.tag/2","doc":"tag(email, \"welcome-email\")\n    tag(email, [\"welcome-email\", \"marketing\"])","ref":"Bamboo.MandrillHelper.html#tag/2-example"},{"type":"function","title":"Bamboo.MandrillHelper.template/3","doc":"Send emails using Mandrill's template API.\n\nSetup Mandrill to send using a named template with template content. Use this\nin conjunction with merge vars to offload template rendering to Mandrill. The\ntemplate name specified here must match the template name stored in Mandrill.\nMandrill's API docs for this can be found [here](https://www.mandrillapp.com/api/docs/messages.JSON.html#method=send-template).","ref":"Bamboo.MandrillHelper.html#template/3"},{"type":"function","title":"Example - Bamboo.MandrillHelper.template/3","doc":"template(email, \"welcome\")\n    template(email, \"welcome\", [%{\"name\" => \"Name\", \"content\" => \"John\"}])","ref":"Bamboo.MandrillHelper.html#template/3-example"},{"type":"module","title":"Bamboo.SendGridAdapter","doc":"Sends email using SendGrid's JSON API.\n\nUse this adapter to send emails through SendGrid's API. Requires that an API\nkey is set in the config.\n\nIf you would like to add a replyto header to your email, then simply pass it in\nusing the header property or put_header function like so:\n\n    put_header(\"reply-to\", \"foo@bar.com\")\n\nTo set arbitrary email headers, set them in the `headers` property of the [Bamboo.Email](Bamboo.Email) struct.\nNote that some header names are reserved for use by Sendgrid. See\n[here](https://sendgrid.com/docs/API_Reference/Web_API_v3/Mail/index.html) for full list.","ref":"Bamboo.SendGridAdapter.html"},{"type":"module","title":"Example config - Bamboo.SendGridAdapter","doc":"# In config/config.exs, or config.prod.exs, etc.\n    config :my_app, MyApp.Mailer,\n      adapter: Bamboo.SendGridAdapter,\n      api_key: \"my_api_key\",\n        # or {:system, \"SENDGRID_API_KEY\"},\n        # or {ModuleName, :method_name, []}\n      hackney_opts: [\n        recv_timeout: :timer.minutes(1)\n      ]\n\n    # To enable sandbox mode (e.g. in development or staging environments),\n    # in config/dev.exs or config/prod.exs etc\n    config :my_app, MyApp.Mailer, sandbox: true\n\n    # Define a Mailer. Maybe in lib/my_app/mailer.ex\n    defmodule MyApp.Mailer do\n      use Bamboo.Mailer, otp_app: :my_app\n    end","ref":"Bamboo.SendGridAdapter.html#module-example-config"},{"type":"function","title":"Bamboo.SendGridAdapter.deliver/2","doc":"","ref":"Bamboo.SendGridAdapter.html#deliver/2"},{"type":"module","title":"Bamboo.SendGridHelper","doc":"Functions for using features specific to Sendgrid.","ref":"Bamboo.SendGridHelper.html"},{"type":"module","title":"Example - Bamboo.SendGridHelper","doc":"email\n    |> with_template(\"80509523-83de-42b6-a2bf-54b7513bd2aa\")\n    |> substitute(\"%name%\", \"Jon Snow\")\n    |> substitute(\"%location%\", \"Westeros\")","ref":"Bamboo.SendGridHelper.html#module-example"},{"type":"function","title":"Bamboo.SendGridHelper.add_dynamic_field/3","doc":"Add a property to the list of dynamic template data in the SendGrid template.\n\nThis will be added to the request as:\n\n```\n \"personalizations\":[\n    {\n       \"to\":[\n          {\n             \"email\":\"example@sendgrid.net\"\n          }\n       ],\n       \"dynamic_template_data\":{\n          \"total\":\"$ 239.85\",\n       }\n    }\n ],\n```\n\nThe tag can be of any type since SendGrid allows you to use Handlebars in its templates","ref":"Bamboo.SendGridHelper.html#add_dynamic_field/3"},{"type":"function","title":"Example - Bamboo.SendGridHelper.add_dynamic_field/3","doc":"email\n    |> add_dynamic_field(\"name\", \"Jon Snow\")","ref":"Bamboo.SendGridHelper.html#add_dynamic_field/3-example"},{"type":"function","title":"Bamboo.SendGridHelper.add_personalizations/2","doc":"Add SendGrid personalizations.\n\nEach personalization can have the following fields: `to`, `cc`, `bcc`,\n`subject`, `headers`, `substitutions`, `custom_args`, or `send_at`.\n\nSettings from the top level of the email (e.g., `Email |> with_send_at`)\nwill not be applied to each personalization.  If you want multiple\npersonalizations with common properties, it is recommended to generate the\nlist from a common base value and simply do not set the corresponding\ntop-level fields.","ref":"Bamboo.SendGridHelper.html#add_personalizations/2"},{"type":"function","title":"Example: - Bamboo.SendGridHelper.add_personalizations/2","doc":"base_personalization = %{\n      bcc: [%{\"email\" => \"bcc@bar.com\", \"name\" => \"BCC\"}],\n      subject: \"Here is your email\"\n    }\n\n    personalizations =\n      Enum.map(\n        [\n          %{to: \"one@test.com\"},\n          %{to: \"two@test.com\", send_at: 1_580_485_560}\n        ],\n        &Map.merge(base_personalization, &1)\n      )\n\n    email =\n      new_email()\n      |> Email.put_header(\"Reply-To\", \"reply@foo.com\")\n      |> Bamboo.SendGridHelper.add_personalizations(personalizations)","ref":"Bamboo.SendGridHelper.html#add_personalizations/2-example"},{"type":"function","title":"Bamboo.SendGridHelper.substitute/3","doc":"Add a tag to the list of substitutions in the SendGrid template.\n\nThe tag must be a `String.t` due to SendGrid using special characters to wrap\ntags in the template.","ref":"Bamboo.SendGridHelper.html#substitute/3"},{"type":"function","title":"Example - Bamboo.SendGridHelper.substitute/3","doc":"email\n    |> substitute(\"%name%\", \"Jon Snow\")","ref":"Bamboo.SendGridHelper.html#substitute/3-example"},{"type":"function","title":"Bamboo.SendGridHelper.with_asm_group_id/2","doc":"Set the ASM (Advanced Suppression Manager) group that this email should belong to.\n\nThis can be used to let recipients unsubscribe from only a certain type of communication.","ref":"Bamboo.SendGridHelper.html#with_asm_group_id/2"},{"type":"function","title":"Example - Bamboo.SendGridHelper.with_asm_group_id/2","doc":"email\n    |> with_asm_group_id(1234)","ref":"Bamboo.SendGridHelper.html#with_asm_group_id/2-example"},{"type":"function","title":"Bamboo.SendGridHelper.with_bypass_list_management/2","doc":"Instruct SendGrid to bypass list management for this email.\n\nIf enabled, SendGrid will ignore any email suppression (such as\nunsubscriptions, bounces, spam filters) for this email. This is useful for\nemails that all users must receive, such as Terms of Service updates, or\npassword resets.","ref":"Bamboo.SendGridHelper.html#with_bypass_list_management/2"},{"type":"function","title":"Example - Bamboo.SendGridHelper.with_bypass_list_management/2","doc":"email\n    |> with_bypass_list_management(true)","ref":"Bamboo.SendGridHelper.html#with_bypass_list_management/2-example"},{"type":"function","title":"Bamboo.SendGridHelper.with_categories/2","doc":"Sets a list of categories for this email.\n\nA maximum of 10 categories can be assigned to an email. Duplicate categories will\nbe ignored and only unique entries will be sent.","ref":"Bamboo.SendGridHelper.html#with_categories/2"},{"type":"function","title":"Example - Bamboo.SendGridHelper.with_categories/2","doc":"email\n    |> with_categories(\"campaign-12345\")","ref":"Bamboo.SendGridHelper.html#with_categories/2-example"},{"type":"function","title":"Bamboo.SendGridHelper.with_click_tracking/2","doc":"Instruct SendGrid to enable or disable Click Tracking for a particular email.\n\nRead more about SendGrid click tracking [here](https://docs.sendgrid.com/ui/account-and-settings/tracking#click-tracking)","ref":"Bamboo.SendGridHelper.html#with_click_tracking/2"},{"type":"function","title":"Example - Bamboo.SendGridHelper.with_click_tracking/2","doc":"email\n    |> with_click_tracking(true)\n\n    email\n    |> with_click_tracking(false)","ref":"Bamboo.SendGridHelper.html#with_click_tracking/2-example"},{"type":"function","title":"Bamboo.SendGridHelper.with_custom_args/2","doc":"Set a map of custom arguments for this email.\n\nThis will override any existing custom arguments.","ref":"Bamboo.SendGridHelper.html#with_custom_args/2"},{"type":"function","title":"Example - Bamboo.SendGridHelper.with_custom_args/2","doc":"email\n    |> with_custom_args(%{new_arg_1: \"new arg 1\", new_arg_2: \"new arg 2\"})","ref":"Bamboo.SendGridHelper.html#with_custom_args/2-example"},{"type":"function","title":"Bamboo.SendGridHelper.with_google_analytics/3","doc":"Instruct SendGrid to enable or disable Google Analytics tracking, and\noptionally set the UTM parameters for it.\n\nThis is useful if you need to control UTM tracking parameters on an individual email\nbasis.","ref":"Bamboo.SendGridHelper.html#with_google_analytics/3"},{"type":"function","title":"Example - Bamboo.SendGridHelper.with_google_analytics/3","doc":"email\n    |> with_google_analytics(true, %{utm_source: \"email\", utm_campaign: \"campaign\"})\n\n    email\n    |> with_google_analytics(false)","ref":"Bamboo.SendGridHelper.html#with_google_analytics/3-example"},{"type":"function","title":"Bamboo.SendGridHelper.with_ip_pool_name/2","doc":"Specify the ip pool name.","ref":"Bamboo.SendGridHelper.html#with_ip_pool_name/2"},{"type":"function","title":"Example - Bamboo.SendGridHelper.with_ip_pool_name/2","doc":"email\n    |> with_ip_pool_name(\"my-ip-pool-name\")","ref":"Bamboo.SendGridHelper.html#with_ip_pool_name/2-example"},{"type":"function","title":"Bamboo.SendGridHelper.with_send_at/2","doc":"Schedule a time for SendGrid to deliver the email.\n\nNote that if the time is in the past, SendGrid will immediately deliver the\nemail.","ref":"Bamboo.SendGridHelper.html#with_send_at/2"},{"type":"function","title":"Example - Bamboo.SendGridHelper.with_send_at/2","doc":"{:ok, delivery_time, _} = DateTime.from_iso8601(\"2020-01-01T00:00:00Z\")\n\n    email\n    |> with_send_at(delivery_time)","ref":"Bamboo.SendGridHelper.html#with_send_at/2-example"},{"type":"function","title":"Bamboo.SendGridHelper.with_template/2","doc":"Specify the template for SendGrid to use for the context of the substitution\ntags.","ref":"Bamboo.SendGridHelper.html#with_template/2"},{"type":"function","title":"Example - Bamboo.SendGridHelper.with_template/2","doc":"with_template(email, \"80509523-83de-42b6-a2bf-54b7513bd2aa\")","ref":"Bamboo.SendGridHelper.html#with_template/2-example"},{"type":"module","title":"Bamboo.SentEmail","doc":"Used for storing and retrieving sent emails when used with `Bamboo.LocalAdapter`.\n\nWhen emails are sent with the `Bamboo.LocalAdapter`, they are stored in\n`Bamboo.SentEmail`. Use the functions in this module to store and retrieve the emails.\n\nRemember to start the Bamboo app by adding it to the app list in `mix.exs` or\nstarting it with `Application.ensure_all_started(:bamboo)`","ref":"Bamboo.SentEmail.html"},{"type":"function","title":"Bamboo.SentEmail.all/0","doc":"Returns a list of all sent emails","ref":"Bamboo.SentEmail.html#all/0"},{"type":"function","title":"Bamboo.SentEmail.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Bamboo.SentEmail.html#child_spec/1"},{"type":"function","title":"Bamboo.SentEmail.get/1","doc":"Gets an email by id. Returns nil if it can't find a matching email.","ref":"Bamboo.SentEmail.html#get/1"},{"type":"function","title":"Bamboo.SentEmail.get!/1","doc":"Gets an email by id. Raises if it can't find one.","ref":"Bamboo.SentEmail.html#get!/1"},{"type":"function","title":"Bamboo.SentEmail.get_id/1","doc":"Gets the email's id.\n\nThe email must be an email that was sent with `Bamboo.LocalAdapter` or added\nvia `Bamboo.SentEmail.push/1`, otherwise the id will not have been set.","ref":"Bamboo.SentEmail.html#get_id/1"},{"type":"function","title":"Bamboo.SentEmail.one/0","doc":"Returns exactly one sent email. Raises if none, or more than one are found\n\nRaises `NoDeliveriesError` if there are no emails. Raises `DeliveriesError` if\nthere are 2 or more emails.","ref":"Bamboo.SentEmail.html#one/0"},{"type":"function","title":"Bamboo.SentEmail.push/1","doc":"Adds an email to the list of sent emails.\n\nAdds an email to the beginning of the sent emails list. Also gives the email\nan id that can be fetched with `Bamboo.SentEmail.get_id/1`.","ref":"Bamboo.SentEmail.html#push/1"},{"type":"function","title":"Bamboo.SentEmail.reset/0","doc":"Clears all sent emails","ref":"Bamboo.SentEmail.html#reset/0"},{"type":"function","title":"Bamboo.SentEmail.start_link/1","doc":"Starts the SentEmail Agent","ref":"Bamboo.SentEmail.html#start_link/1"},{"type":"module","title":"Bamboo.SentEmailApiPlug","doc":"A plug that exposes delivered emails over a JSON API.\n\nThis is useful for an integration test runner that doesn't have\naccess to the `Bamboo.Test` helpers, but needs to be able to assert that\nemails have been delivered.\n\nYou must use the `Bamboo.LocalAdapter` or no emails will be available.","ref":"Bamboo.SentEmailApiPlug.html"},{"type":"module","title":"Using with Plug or Phoenix - Bamboo.SentEmailApiPlug","doc":"# Make sure you are using Bamboo.LocalAdapter in your config\n    config :my_app, MyApp.Mailer,\n      adapter: Bamboo.LocalAdapter\n\n    # In your Router\n    defmodule MyApp.Router do\n      use Phoenix.Router # or use Plug.Router if you're not using Phoenix\n\n      if Mix.env == :e2e do\n        # If using Phoenix\n        forward \"/sent_emails_api\", Bamboo.SentEmailApiPlug\n\n        # If using Plug.Router, make sure to add the `to`\n        forward \"/sent_emails_api\", to: Bamboo.SentEmailApiPlug\n      end\n    end","ref":"Bamboo.SentEmailApiPlug.html#module-using-with-plug-or-phoenix"},{"type":"module","title":"API endpoints - Bamboo.SentEmailApiPlug","doc":"There are two endpoints:\n\n* `GET /emails.json` exposes the list of sent emails\n* `POST /reset.json` resets the sent emails back to an empty list\n\nUsually, you would `POST` to `/reset.json` at the beginning of a test run,\nand `GET` from `/emails.json` as you need to during an integration test to\nmake assertions. The details of how you make assertions is left up to your\nintegration test runner.","ref":"Bamboo.SentEmailApiPlug.html#module-api-endpoints"},{"type":"module","title":"Email response JSON - Bamboo.SentEmailApiPlug","doc":"Sent emails are returned in the following format:\n\n```json\n{\n  \"to\": [[null, \"to@example.com\"], [\"Alice\", \"alice@example.com\"]],\n  \"text_body\": \"hello world\",\n  \"subject\": \"This is a test email\",\n  \"html_body\": \" hello world \",\n  \"headers\": {},\n  \"from\": [null, \"from@example.com\"],\n  \"cc\": [[null, \"cc@example.com\"]],\n  \"bcc\": [[\"Bob\", \"bob@example.com\"]],\n  \"attachments\": []\n}\n```\n\nEmail addresses are returned in array pairs, with the first element as the\nname, and the second as the email address. The first element can be `null`\nif the address didn't specify a name. The `to`, `cc` and `bcc` fields are\nreturned as arrays of these pairs.","ref":"Bamboo.SentEmailApiPlug.html#module-email-response-json"},{"type":"function","title":"Bamboo.SentEmailApiPlug.call/2","doc":"","ref":"Bamboo.SentEmailApiPlug.html#call/2"},{"type":"function","title":"Bamboo.SentEmailApiPlug.init/1","doc":"","ref":"Bamboo.SentEmailApiPlug.html#init/1"},{"type":"module","title":"Bamboo.SentEmailViewerPlug","doc":"A plug that can be used in your router to see delivered emails.\n\nThis plug allows you to view all delivered emails. To see emails you must use\nthe `Bamboo.LocalAdapter`.","ref":"Bamboo.SentEmailViewerPlug.html"},{"type":"module","title":"Using with Plug or Phoenix - Bamboo.SentEmailViewerPlug","doc":"# Make sure you are using Bamboo.LocalAdapter in your config\n    config :my_app, MyApp.Mailer,\n      adapter: Bamboo.LocalAdapter\n\n    # In your Router\n    defmodule MyApp.Router do\n      use Phoenix.Router # or use Plug.Router if you're not using Phoenix\n\n      if Mix.env == :dev do\n        # If using Phoenix\n        forward \"/sent_emails\", Bamboo.SentEmailViewerPlug\n\n        # If using Plug.Router, make sure to add the `to`\n        forward \"/sent_emails\", to: Bamboo.SentEmailViewerPlug\n      end\n    end\n\nNow if you visit your app at `/sent_emails` you will see a list of delivered\nemails.","ref":"Bamboo.SentEmailViewerPlug.html#module-using-with-plug-or-phoenix"},{"type":"function","title":"Bamboo.SentEmailViewerPlug.call/2","doc":"","ref":"Bamboo.SentEmailViewerPlug.html#call/2"},{"type":"function","title":"Bamboo.SentEmailViewerPlug.init/1","doc":"","ref":"Bamboo.SentEmailViewerPlug.html#init/1"},{"type":"module","title":"Bamboo.TaskSupervisorStrategy","doc":"Default strategy. Sends an email in the background using `Task.Supervisor`.\n\nThis is the default strategy when calling `deliver_later` because it is the\nsimplest to get started with. This strategy uses a `Task.Supervisor` to monitor\nthe delivery. Deliveries that fail will raise, but will not be retried, and\nwill not bring down the calling process.","ref":"Bamboo.TaskSupervisorStrategy.html"},{"type":"module","title":"Why use it? - Bamboo.TaskSupervisorStrategy","doc":"Sending emails can often take time and may fail. If you are sending email\nduring a web request (for instance, sending a welcome email), you probably\ndon't want to make your users wait the extra time for the welcome email to send.\nInstead you can use `deliver_later/1` and it will be delivered in the background\nso web requests remain snappy.","ref":"Bamboo.TaskSupervisorStrategy.html#module-why-use-it"},{"type":"function","title":"Bamboo.TaskSupervisorStrategy.supervisor_name/0","doc":"","ref":"Bamboo.TaskSupervisorStrategy.html#supervisor_name/0"},{"type":"module","title":"Bamboo.Template","doc":"Render emails with layouts, view modules, and templates.\n\nThis module allows rendering emails with layouts and views. Pass an\natom (e.g. `:welcome_email`) as the template name to render both HTML and\nplain text emails. Use a string if you only want to render one type, e.g.\n`\"welcome_email.text\"` or `\"welcome_email.html\"`.","ref":"Bamboo.Template.html"},{"type":"module","title":"Examples - Bamboo.Template","doc":"_Set the text and HTML layout for an email._\n\n    defmodule MyApp.Email do\n      use Bamboo.Template, view: MyApp.Email.AccountView\n\n      def welcome_email do\n        new_email()\n        |> put_text_layout({MyApp.Email.LayoutView, \"email.text\"})\n        |> put_html_layout({MyApp.Email.LayoutView, \"email.html\"})\n        |> render(:welcome) # Pass atom to render html AND plain text templates\n      end\n    end\n\n_You can optionally pass the html and text layouts when calling `use Bamboo.Template`_\n\n    defmodule MyApp.Email do\n      use Bamboo.Template,\n        view: MyApp.Email.AccountView\n        text_layout: {MyApp.Email.LayoutView, \"email.text\"},\n        html_layout: {MyApp.Email.LayoutView, \"email.html\"}\n\n      def welcome_email do\n        new_email()\n        |> render(:welcome) # Pass atom to render html AND plain text templates\n      end\n    end\n\n_Set both the text and HTML layout at the same time for an email._\n\n    defmodule MyApp.Email do\n      use Bamboo.Template, view: MyApp.Email.AccountView\n\n      def welcome_email do\n        new_email()\n        |> put_layout({MyApp.Email.LayoutView, :email})\n        |> render(:welcome)\n      end\n    end\n\n_Render both text and html emails without layouts._\n\n    defmodule MyApp.Email do\n      use Bamboo.Template, view: MyApp.Email.AccountView\n\n      def welcome_email do\n        new_email()\n        |> render(:welcome)\n      end\n    end\n\n_Make assigns available to a template._\n\n    defmodule MyApp.Email do\n      use Bamboo.Template, view: MyApp.Email.AccountView\n\n      def welcome_email(user) do\n        new_email()\n        |> assign(:user, user)\n        |> render(:welcome)\n      end\n    end\n\n_Make assigns available to a template during render call._\n\n    defmodule MyApp.Email do\n      use Bamboo.Template, view: MyApp.Email.AccountView\n\n      def welcome_email(user) do\n        new_email()\n        |> render(:welcome, user: user)\n      end\n    end\n\n_Render an email by passing the template string to render._\n\n    defmodule MyApp.Email do\n      use Bamboo.Template, view: MyApp.Email.AccountView\n\n      def html_email do\n        new_email()\n        |> render(\"html_email.html\")\n      end\n\n      def text_email do\n        new_email\n        |> render(\"text_email.text\")\n      end\n    end","ref":"Bamboo.Template.html#module-examples"},{"type":"module","title":"HTML Layout Example - Bamboo.Template","doc":"# lib/my_app/email.ex\n    defmodule MyApp.Email do\n      use Bamboo.Template, view: MyApp.Email.AccountView\n\n      def welcome_email(person) do\n        base_email()\n        |> to(person)\n        |> subject(\"Welcome to MyApp\")\n        |> assign(:person, person)\n        |> render(:welcome_email)\n      end\n\n      defp base_email do\n        new_email()\n        |> from(\"Rob Ot \")\n        |> put_header(\"Reply-To\", \"editors@changelog.com\")\n        # This will use the \"email.html.eex\" file as a layout when rendering html emails.\n        # Plain text emails will not use a layout unless you use `put_text_layout`\n        |> put_html_layout({MyApp.Email.LayoutView, \"email.html\"})\n      end\n    end\n\n    # lib/my_app/email/views/layout_view.ex\n    defmodule MyApp.Email.LayoutView do\n      use Bamboo.View, path: \"lib/my_app/email/templates/layout\"\n    end\n\n    # lib/my_app/email/templates/layout/email.html.eex\n     \n       \n        <%= @inner_content %>\n       \n     \n\n    # lib/my_app/email/views/account_view.ex\n    defmodule MyApp.Email.AccountView do\n      use Bamboo.View, path: \"lib/my_app/email/templates/account\"\n    end\n\n    # lib/my_app/email/templates/account/welcome_email.html.eex\n    # This will be rendered within a layout because `put_html_layout` was used.\n     Welcome <%= @person.name %> \n\n    # lib/my_app/email/templates/account/welcome_email.text.eex\n    # This will not be rendered within a layout because `put_text_layout` was not used.\n    Welcome <%= @person.name %>","ref":"Bamboo.Template.html#module-html-layout-example"},{"type":"function","title":"Bamboo.Template.assign/2","doc":"Sets many assigns for an email. Accepts a map or a keyword list. See\n`assign/3` for more.","ref":"Bamboo.Template.html#assign/2"},{"type":"function","title":"Example - Bamboo.Template.assign/2","doc":"new_email()\n    |> assign(%{user: user})\n    |> render(:template_name)\n\n    new_email()\n    |> assign(user: user)\n    |> render(:template_name)","ref":"Bamboo.Template.html#assign/2-example"},{"type":"function","title":"Bamboo.Template.assign/3","doc":"Sets an assign for the email. These will be available when rendering the email.","ref":"Bamboo.Template.html#assign/3"},{"type":"function","title":"Example - Bamboo.Template.assign/3","doc":"new_email()\n    # assigns user to be accessed as `@user` in template\n    |> assign(:user, user)\n    |> render(:template_name)","ref":"Bamboo.Template.html#assign/3-example"},{"type":"function","title":"Bamboo.Template.put_html_layout/2","doc":"Sets the layout when rendering HTML templates.","ref":"Bamboo.Template.html#put_html_layout/2"},{"type":"function","title":"Example - Bamboo.Template.put_html_layout/2","doc":"def html_email_layout do\n      new_email\n      # Will use the email.html template of MyApp.LayoutView when rendering html emails\n      |> put_html_layout({MyApp.LayoutView, \"email.html\"})\n    end","ref":"Bamboo.Template.html#put_html_layout/2-example"},{"type":"function","title":"Bamboo.Template.put_layout/2","doc":"Sets the layout for rendering plain text and HTML templates.","ref":"Bamboo.Template.html#put_layout/2"},{"type":"function","title":"Example - Bamboo.Template.put_layout/2","doc":"def text_and_html_email_layout do\n      new_email\n      # Will use email.html and email.text templates of MyApp.LayoutView\n      # when rendering HTML and text emails\n      |> put_layout({MyAppWeb.LayoutView, :email})\n    end","ref":"Bamboo.Template.html#put_layout/2-example"},{"type":"function","title":"Bamboo.Template.put_text_layout/2","doc":"Sets the layout when rendering plain text templates.","ref":"Bamboo.Template.html#put_text_layout/2"},{"type":"function","title":"Example - Bamboo.Template.put_text_layout/2","doc":"def text_email_layout do\n      new_email\n      # Will use the email.text template of MyApp.LayoutView when rendering text emails\n      |> put_text_layout({MyApp.LayoutView, \"email.text\"})\n    end","ref":"Bamboo.Template.html#put_text_layout/2-example"},{"type":"function","title":"Bamboo.Template.put_view/2","doc":"Overrides the view for rendering templates","ref":"Bamboo.Template.html#put_view/2"},{"type":"function","title":"Example - Bamboo.Template.put_view/2","doc":"defmodule MyApp.Email do\n      use Bamboo.Template, view: MyApp.EmailView\n\n      def different_view_template do\n        new_email\n        # Will use welcome.html template of MyApp.AccountView\n        |> put_view(MyApp.AccountView)\n        |> render(\"welcome.html\")\n      end\n    end","ref":"Bamboo.Template.html#put_view/2-example"},{"type":"function","title":"Bamboo.Template.render/3","doc":"Render a template and set the body on the email.\n\nPass an atom as the template name to render HTML *and* plain text emails,\ne.g. `:welcome_email`. Use a string if you only want to render one type, e.g.\n`\"welcome_email.text\"` or `\"welcome_email.html\"`.\n\nYou can also optionally pass assigns.","ref":"Bamboo.Template.html#render/3"},{"type":"function","title":"Example - Bamboo.Template.render/3","doc":"# renders both HTML and text emails\n    render(new_email(), :template_name)\n\n    # renders HTML template\n    render(new_email(), \"template_name.html\")\n\n    # renders text template\n    render(new_email(), \"template_name.text\")\n\n    # renders with assigns\n    render(new_email(), :template_name, user: user)","ref":"Bamboo.Template.html#render/3-example"},{"type":"module","title":"Bamboo.Test","doc":"Helpers for testing email delivery.\n\nUse these helpers with `Bamboo.TestAdapter` to test email delivery. Typically\nyou'll want to **unit test emails first**. Then, in integration tests, use\nhelpers from this module to test whether that email was delivered.","ref":"Bamboo.Test.html"},{"type":"module","title":"Note on sending from other processes - Bamboo.Test","doc":"If you are sending emails from another process (for example, from inside a\nTask or GenServer) you may need to use shared mode when using\n`Bamboo.Test`. See the docs `__using__/1` for an example.\n\nFor most scenarios you will not need shared mode.","ref":"Bamboo.Test.html#module-note-on-sending-from-other-processes"},{"type":"module","title":"In your config - Bamboo.Test","doc":"# Typically in config/test.exs\n    config :my_app, MyApp.Mailer,\n      adapter: Bamboo.TestAdapter","ref":"Bamboo.Test.html#module-in-your-config"},{"type":"module","title":"Unit test - Bamboo.Test","doc":"You don't need any special functions to unit test emails.\n\n    defmodule MyApp.EmailTest do\n      use ExUnit.Case\n\n      test \"welcome email\" do\n        user = %User{name: \"John\", email: \"person@example.com\"}\n\n        email = MyApp.Email.welcome_email(user)\n\n        assert email.to == user\n        assert email.subject == \"This is your welcome email\"\n        assert email.html_body =~ \"Welcome to the app\"\n      end\n    end","ref":"Bamboo.Test.html#module-unit-test"},{"type":"module","title":"Integration test - Bamboo.Test","doc":"defmodule MyApp.Email do\n      import Bamboo.Email\n\n      def welcome_email(user) do\n        new_email(\n          from: \"me@app.com\",\n          to: user,\n          subject: \"Welcome!\",\n          text_body: \"Welcome to the app\",\n          html_body: \" Welcome to the app \"\n        )\n      end\n    end\n\n    defmodule MyApp.EmailDeliveryTest do\n      use ExUnit.Case\n      use Bamboo.Test\n\n      test \"sends welcome email\" do\n        user = %User{...}\n        email = MyApp.Email.welcome_email(user)\n\n        email |> MyApp.Mailer.deliver_now\n\n        # Works with deliver_now and deliver_later\n        assert_delivered_email email\n      end\n    end","ref":"Bamboo.Test.html#module-integration-test"},{"type":"macro","title":"Bamboo.Test.__using__/1","doc":"Imports `Bamboo.Test` and `Bamboo.Formatter.format_email_address/2`\n\n`Bamboo.Test` and the `Bamboo.TestAdapter` work by sending a message to the\ncurrent process when an email is delivered. The process mailbox is then\nchecked when using the assertion helpers like `assert_delivered_email/1`.\n\nSometimes emails don't show up when asserting because you may deliver an email\nfrom a _different_ process than the test process. When that happens, turn on\nshared mode. This will tell `Bamboo.TestAdapter` to always send to the test process.\nThis means that you cannot use shared mode with async tests.","ref":"Bamboo.Test.html#__using__/1"},{"type":"macro","title":"Try to use this version first - Bamboo.Test.__using__/1","doc":"use Bamboo.Test","ref":"Bamboo.Test.html#__using__/1-try-to-use-this-version-first"},{"type":"macro","title":"And if you are delivering from another process, set `shared: true` - Bamboo.Test.__using__/1","doc":"use Bamboo.Test, shared: true\n\nCommon scenarios for delivering mail from a different process are when you\nsend from inside of a Task, GenServer, or are running acceptance tests with a\nheadless browser like phantomjs.","ref":"Bamboo.Test.html#__using__/1-and-if-you-are-delivering-from-another-process-set-shared-true"},{"type":"function","title":"Bamboo.Test.assert_delivered_email/2","doc":"Checks whether an email was delivered.\n\nMust be used with the `Bamboo.TestAdapter` or this will never pass. In case you\nare delivering from another process, the assertion waits up to 100ms before\nfailing. Typically if an email is successfully delivered the assertion will\npass instantly, so test suites will remain fast.","ref":"Bamboo.Test.html#assert_delivered_email/2"},{"type":"function","title":"Examples - Bamboo.Test.assert_delivered_email/2","doc":"email = Bamboo.Email.new_email(subject: \"something\")\n    email |> MyApp.Mailer.deliver\n    assert_delivered_email(email) # Will pass\n\n    unsent_email = Bamboo.Email.new_email(subject: \"something else\")\n    assert_delivered_email(unsent_email) # Will fail","ref":"Bamboo.Test.html#assert_delivered_email/2-examples"},{"type":"macro","title":"Bamboo.Test.assert_delivered_email_matches/2","doc":"Checks whether an email was delivered matching the given pattern.\n\nMust be used with the `Bamboo.TestAdapter` or this will never pass. This\nallows the user to use their configured `assert_receive_timeout` for ExUnit,\nand also to match any variables in their given pattern for use in further\nassertions.","ref":"Bamboo.Test.html#assert_delivered_email_matches/2"},{"type":"macro","title":"Examples - Bamboo.Test.assert_delivered_email_matches/2","doc":"%{email: user_email, name: user_name} = user\n    MyApp.deliver_welcome_email(user)\n    assert_delivered_email_matches(%{to: [{_, ^user_email}], text_body: text_body})\n    assert text_body =~ \"Welcome to MyApp, #{user_name}\"\n    assert text_body =~ \"You can sign up at https://my_app.com/users/#{user_name}\"","ref":"Bamboo.Test.html#assert_delivered_email_matches/2-examples"},{"type":"function","title":"Bamboo.Test.assert_email_delivered_with/2","doc":"Check whether an email's params are equal to the ones provided.\n\nMust be used with the `Bamboo.TestAdapter` or this will never pass. In case you\nare delivering from another process, the assertion waits up to 100ms before\nfailing. Typically if an email is successfully delivered the assertion will\npass instantly, so test suites will remain fast.","ref":"Bamboo.Test.html#assert_email_delivered_with/2"},{"type":"function","title":"Examples - Bamboo.Test.assert_email_delivered_with/2","doc":"email = Bamboo.Email.new_email(subject: \"something\")\n    email |> MyApp.Mailer.deliver\n    assert_email_delivered_with(subject: \"something\") # Will pass\n\n    unsent_email = Bamboo.Email.new_email(subject: \"something else\")\n    assert_email_delivered_with(subject: \"something else\") # Will fail\n\nThe function will use the Bamboo Formatter when checking email addresses.\n\n    email = Bamboo.Email.new_email(to: \"someone@example.com\")\n    email |> MyApp.Mailer.deliver\n    assert_email_delivered_with(to: \"someone@example.com\") # Will pass\n\nYou can also pass a regex to match portions of an email.","ref":"Bamboo.Test.html#assert_email_delivered_with/2-examples"},{"type":"function","title":"Example - Bamboo.Test.assert_email_delivered_with/2","doc":"email = new_email(text_body: \"I love coffee\")\n    email |> MyApp.Mailer.deliver\n    assert_email_delivered_with(text_body: ~r/love/) # Will pass\n    assert_email_delivered_with(text_body: ~r/like/) # Will fail","ref":"Bamboo.Test.html#assert_email_delivered_with/2-example"},{"type":"function","title":"Bamboo.Test.assert_no_emails_delivered/1","doc":"Checks that no emails were sent.\n\nIf `Bamboo.Test` is used with shared mode, you must also configure a timeout\nin your test config.\n\n    # Set this in your config, typically in config/test.exs\n    config :bamboo, :refute_timeout, 10\n\nThe value you set is up to you. Lower values will result in faster tests,\nbut may incorrectly pass if an email is delivered *after* the timeout. Often\ntimes 1ms is enough.","ref":"Bamboo.Test.html#assert_no_emails_delivered/1"},{"type":"function","title":"Bamboo.Test.refute_delivered_email/2","doc":"Ensures a particular email was not sent\n\nSame as `assert_delivered_email/0`, except it checks that a particular email\nwas not sent.\n\nIf `Bamboo.Test` is used with shared mode, you must also configure a timeout\nin your test config.\n\n    # Set this in your config, typically in config/test.exs\n    config :bamboo, :refute_timeout, 10\n\nThe value you set is up to you. Lower values will result in faster tests,\nbut may incorrectly pass if an email is delivered *after* the timeout. Often\ntimes 1ms is enough.","ref":"Bamboo.Test.html#refute_delivered_email/2"},{"type":"function","title":"Bamboo.Test.refute_email_delivered_with/2","doc":"Check that no email was sent with the given parameters.\n\nSimilar to `assert_email_delivered_with/1`, but it checks that an email with\nthose parameters wasn't sent.\n\nNote that this assertion helper will grab the email out of the process\nmailbox. So if you want to make other assertions about the same email after\nthis assertion, you need to send the email again.","ref":"Bamboo.Test.html#refute_email_delivered_with/2"},{"type":"function","title":"Examples - Bamboo.Test.refute_email_delivered_with/2","doc":"Bamboo.Email.new_email(subject: \"something\") |> MyApp.Mailer.deliver()\n    refute_email_delivered_with(subject: \"something else\") # Will pass\n\n    Bamboo.Email.new_email(subject: \"something\") |> MyApp.Mailer.deliver()\n    refute_email_delivered_with(subject: ~r/some/) # Will fail\n\nIf `Bamboo.Test` is used with shared mode, you must also configure a timeout\nin your test config.\n\n    # Set this in your config, typically in config/test.exs\n    config :bamboo, :refute_timeout, 10\n\nThe value you set is up to you. Lower values may result in faster tests, but\nyour tests may incorrectly pass if an email is delivered *after* the timeout.\n\nYou can also pass a timeout for a given refutation:","ref":"Bamboo.Test.html#refute_email_delivered_with/2-examples"},{"type":"function","title":"Examples - Bamboo.Test.refute_email_delivered_with/2","doc":"Bamboo.Email.new_email(subject: \"something\") |> MyApp.Mailer.deliver()\n    refute_email_delivered_with([subject: \"something else\"], timeout: 100)","ref":"Bamboo.Test.html#refute_email_delivered_with/2-examples"},{"type":"module","title":"Bamboo.TestAdapter","doc":"Used for testing email delivery.\n\nNo emails are sent, instead a message is sent to the current process and can\nbe asserted on with helpers from `Bamboo.Test`.","ref":"Bamboo.TestAdapter.html"},{"type":"module","title":"Example config - Bamboo.TestAdapter","doc":"# Typically done in config/test.exs\n    config :my_app, MyApp.Mailer,\n      adapter: Bamboo.TestAdapter\n\n    # Define a Mailer. Typically in lib/my_app/mailer.ex\n    defmodule MyApp.Mailer do\n      use Bamboo.Mailer, otp_app: :my_app\n    end","ref":"Bamboo.TestAdapter.html#module-example-config"},{"type":"function","title":"Bamboo.TestAdapter.handle_config/1","doc":"","ref":"Bamboo.TestAdapter.html#handle_config/1"},{"type":"module","title":"Bamboo.View","doc":"Compiles and renders templates defined in a given path.\n\nFunctions defined in the view are available to use in its templates.","ref":"Bamboo.View.html"},{"type":"module","title":"Example - Bamboo.View","doc":"defmodule MyApp.EmailView do\n      use Bamboo.View, path: \"lib/my_app/email/templates\"\n\n      def app_title do\n        \"My Super App\"\n      end\n    end\n\n    # lib/my_app/email_templates/welcome.html\n     Welcome to <%= app_title() %>","ref":"Bamboo.View.html#module-example"},{"type":"exception","title":"Bamboo.ApiError","doc":"Error used to represent a problem when sending emails through an external email service API.","ref":"Bamboo.ApiError.html"},{"type":"function","title":"Bamboo.ApiError.build_api_error/1","doc":"","ref":"Bamboo.ApiError.html#build_api_error/1"},{"type":"function","title":"Bamboo.ApiError.build_api_error/4","doc":"","ref":"Bamboo.ApiError.html#build_api_error/4"},{"type":"function","title":"Bamboo.ApiError.raise_api_error/1","doc":"Raises an `ApiError` with the given `message` or `service_name`, `response` and `params`.\n\nAn extra error message can be added using a fourth parameter `extra_message`.","ref":"Bamboo.ApiError.html#raise_api_error/1"},{"type":"function","title":"Examples - Bamboo.ApiError.raise_api_error/1","doc":"iex> raise_api_error(\"Error message\")\n    ** (Bamboo.ApiError) Error Message\n\n    iex> raise_api_error(service_name, response, params)\n    ** (Bamboo.ApiError) There was a problem sending the email through the   API.\n\n    Here is the response:\n\n    \" \"\n\n    Here are the params we sent:\n\n    \" \"\n\n    iex> raise_api_error(service_name, response, params, extra_message)\n    ** (Bamboo.ApiError) There was a problem sending the email through the   API.\n\n    Here is the response:\n\n    \" \"\n\n    Here are the params we sent:\n\n    \" \"","ref":"Bamboo.ApiError.html#raise_api_error/1-examples"},{"type":"function","title":"Bamboo.ApiError.raise_api_error/4","doc":"","ref":"Bamboo.ApiError.html#raise_api_error/4"},{"type":"exception","title":"Bamboo.EmptyFromAddressError","doc":"","ref":"Bamboo.EmptyFromAddressError.html"},{"type":"exception","title":"Bamboo.NilRecipientsError","doc":"","ref":"Bamboo.NilRecipientsError.html"},{"type":"exception","title":"Bamboo.SentEmail.DeliveriesError","doc":"","ref":"Bamboo.SentEmail.DeliveriesError.html"},{"type":"exception","title":"Bamboo.SentEmail.NoDeliveriesError","doc":"","ref":"Bamboo.SentEmail.NoDeliveriesError.html"},{"type":"exception","title":"Bamboo.View.UndefinedTemplateError","doc":"Exception raised when a template cannot be found.","ref":"Bamboo.View.UndefinedTemplateError.html"},{"type":"function","title":"Bamboo.View.UndefinedTemplateError.message/1","doc":"","ref":"Bamboo.View.UndefinedTemplateError.html#message/1"},{"type":"extras","title":"README","doc":"![bamboo](https://user-images.githubusercontent.com/22394/39895001-b13a9c9a-5476-11e8-9c58-f5fc5f09b697.png)\n\n# Bamboo\n\n[![Build](https://github.com/beam-community/bamboo/actions/workflows/ci.yaml/badge.svg)](https://github.com/beam-community/bamboo/actions/workflows/ci.yaml)\n\n---\n\n**This README follows the main branch, which may not be the currently published version!** Use\n[the docs for the published version of Bamboo](https://hexdocs.pm/bamboo/readme.html).\n\n---\n\n**Bamboo is now maintained by the [BEAM Community](https://github.com/beam-community)!**\n\nThank you to [thoughtbot](https://thoughtbot.com) for creating and maintaining Bamboo for so long!\n\nFlexible and easy to use email for Elixir.\n\n- **Built-in support for popular mail delivery services**. Bamboo ships with\n  [adapters for several popular mail delivery services, including Mandrill,\n  Mailgun, and SendGrid][available-adapters]. It's also quite easy to write\n  your own delivery adapter if your platform isn't yet supported.\n- **Deliver emails in the background**. Most of the time you don't want or need\n  to wait for the email to send. Bamboo makes it easy with\n  `Mailer.deliver_later`.\n- **A functional approach to mail delivery**. Emails are created, manipulated,\n  and sent using plain functions. This makes composition a breeze and fits\n  naturally into your existing Elixir app.\n- **Unit test with ease**. Bamboo separates email creation and email delivery\n  allowing you to test by asserting against email fields without the need for\n  special functions.\n- **Dead-simple integration tests**. Bamboo provides helper functions to make\n  integration testing easy and robust.\n- **View sent emails during development**. Bamboo provides a plug you can use\n  in your router to view sent emails.\n- **Integrate with Phoenix out of the box**. Use Phoenix views and layouts to\n  make rendering email easy.\n\nSee the [docs] for the most up to date information.\n\nWe designed Bamboo to be simple and powerful. If you run into _anything_ that\nis less than exceptional, or you just need some help, please open an issue.","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"To install Bamboo, add it to your list of dependencies in `mix.exs`.\n\n```elixir\ndef deps do\n  [{:bamboo, \"~> 2.3.0\"}]\nend\n```\n\nYou may also use the latest code available from master instead of a\npublished version in hex:\n\n```elixir\ndef deps do\n  [{:bamboo, github: \"beam-community/bamboo\"}]\nend\n```\n\nOnce you've added Bamboo to your list, update your dependencies by running:\n\n```sh\n$ mix deps.get\n```\n\nIf you are using Elixir < 1.4, also ensure Bamboo is started alongside your\napplication:\n\n```elixir\ndef application do\n  [applications: [:bamboo]]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Getting Started - README","doc":"Bamboo separates the tasks of email creation and email sending. To use Bamboo,\nyou'll need to define one or more email modules (email creation), define a\nmailer module (email sending), and provide some configuration.\n\nTo create emails, define an email module within your application.\n\n```elixir\n# some/path/within/your/app/email.ex\ndefmodule MyApp.Email do\n  import Bamboo.Email\n\n  def welcome_email do\n    new_email(\n      to: \"john@example.com\",\n      from: \"support@myapp.com\",\n      subject: \"Welcome to the app.\",\n      html_body: \" Thanks for joining! \",\n      text_body: \"Thanks for joining!\"\n    )\n  end\nend\n```\n\nIn addition to the keyword syntax above you can also [compose emails using pipes].\n\nTo send emails, define a mailer module for your application that `use`s\nBamboo's mailer.\n\n```elixir\n# some/path/within/your/app/mailer.ex\ndefmodule MyApp.Mailer do\n  use Bamboo.Mailer, otp_app: :my_app\nend\n```\n\nYour configuration will need to know your OTP application, your mailer module,\nthe adapter you are using, and any additional configuration required by the\nadapter itself.\n\n```elixir\n# config/config.exs\nconfig :my_app, MyApp.Mailer,\n  adapter: Bamboo.MandrillAdapter,\n  api_key: \"my_api_key\"\n```\n\nBamboo uses [Hackney](https://github.com/benoitc/hackney) for making requests.\nIf you want to pass options to Hackney directly, such as controlling\ntimeouts, you can use the `hackney_opts` key:\n\n```elixir\n# config/config.exs\nconfig :my_app, MyApp.Mailer,\n  adapter: Bamboo.MandrillAdapter,\n  api_key: \"my_api_key\",\n  hackney_opts: [\n    recv_timeout: :timer.minutes(1),\n    connect_timeout: :timer.minutes(1)\n  ]\n```\n\n_Other adapter-specific configuration may be required. Be sure to check the\nadapter's docs._\n\nNow that you have configured Bamboo and defined your modules, you can deliver\nemail in fitting places within your application.\n\n```elixir\ndefmodule MyApp.SomeControllerPerhaps do\n  def send_welcome_email do\n    Email.welcome_email()   # Create your email\n    |> Mailer.deliver_now!() # Send your email\n  end\nend\n```\n\nYour application is now set up to send email with Bamboo! :tada:","ref":"readme.html#getting-started"},{"type":"extras","title":"Using Adapters - README","doc":"An adapter is a set of instructions for how to communicate with a specific\nemail delivery service. Bamboo ships with support for [several popular\nservices][available-adapters], there are others made available by the\ncommunity, or you can use other services by writing a custom adapter.\n\nTo use an adapter, declare it in the configuration for your mailer:\n\n```elixir\n# config/config.exs\nconfig :my_app, MyApp.Mailer,\n  adapter: Bamboo.MandrillAdapter\n```\n\nBamboo provides adapters for use in development and testing. To use these\nadapters, declare them in the environment configuration.\n\nThe local adapter [stores emails in memory that can be viewed during\ndevelopment](#viewing-sent-emails). Declare its use in your dev environment.\n\n```elixir\n# config/dev.exs\nconfig :my_app, MyApp.Mailer,\n  adapter: Bamboo.LocalAdapter\n```\n\nThe test adapter sends emails to your running process allowing you to test mail\ndelivery without emails being sent externally. Declare its use in your test\nenvironment.\n\n```elixir\n# config/test.exs\nconfig :my_app, MyApp.Mailer,\n  adapter: Bamboo.TestAdapter\n```\n\nYou can create new adapters for any environment by implementing the\n[`Bamboo.Adapter`] behaviour.","ref":"readme.html#using-adapters"},{"type":"extras","title":"Delivering Emails in the Background - README","doc":"Often times you don't want to send an email right away because it can block\nprocess completion (e.g. a web request in Phoenix). Bamboo provides a\n`deliver_later` function on your mailers to send emails in the background. It\nalso provides a [`Bamboo.DeliverLaterStrategy`] behaviour that you can\nimplement to tailor your background email sending.\n\nBy default, `deliver_later` uses [`Bamboo.TaskSupervisorStrategy`]. This\nstrategy sends the email right away, but it does so in the background without\nlinking to the calling process, so errors in the mailer won't bring down your\napp.\n\nYou can also create custom strategies by implementing the\n[`Bamboo.DeliverLaterStrategy`] behaviour. For example, you could create\nstrategies for adding emails to a background processing queue such as [exq] or\n[toniq].","ref":"readme.html#delivering-emails-in-the-background"},{"type":"extras","title":"Composing with Pipes - README","doc":"In addition to creating emails with keyword lists you, can use pipe syntax to\ncompose emails. This is particularly useful for providing defaults (e.g. from\naddress, default layout, etc.)\n\n```elixir\ndefmodule MyApp.Email do\n  import Bamboo.Email\n  import Bamboo.Phoenix\n\n  def welcome_email do\n    base_email() # Build your default email then customize for welcome\n    |> to(\"foo@bar.com\")\n    |> subject(\"Welcome!!!\")\n    |> put_header(\"Reply-To\", \"someone@example.com\")\n    |> html_body(\" Welcome \")\n    |> text_body(\"Welcome\")\n  end\n\n  defp base_email do\n    new_email()\n    |> from(\"myapp@example.com\") # Set a default from\n    |> put_html_layout({MyApp.LayoutView, \"email.html\"}) # Set default layout\n    |> put_text_layout({MyApp.LayoutView, \"email.text\"}) # Set default text layout\n  end\nend\n```","ref":"readme.html#composing-with-pipes"},{"type":"extras","title":"Handling Recipients - README","doc":"The from, to, cc, and bcc addresses can be a string or a 2 element tuple. What\nhappens if you try to send to a list of `MyApp.User`s? Transforming your data\nstructure each time you send an email would be a pain.\n\n```elixir\n# This stinks. Do you want to do this every time you create a new email?\nusers = for user <- users do\n  {user.name, user.email}\nend\n\nnew_email(to: users)\n```\n\nBamboo alleviates this pain by providing the [`Bamboo.Formatter`] protocol. By\nimplementing the protocol for your data structure once, you can pass that\nstruct directly to Bamboo anywhere it expects an address. See the\n[`Bamboo.Email`] and [`Bamboo.Formatter`] docs for more information and\nexamples.","ref":"readme.html#handling-recipients"},{"type":"extras","title":"Interceptors - README","doc":"It's possible to configure per Mailer interceptors. Interceptors allow you to\nmodify or block emails on the fly.\n\n```elixir\n# config/config.exs\nconfig :my_app, MyApp.Mailer,\n  adapter: Bamboo.MandrillAdapter,\n  interceptors: [MyApp.DenyListInterceptor]\nend\n```\n\nAn interceptor must implement the `Bamboo.Interceptor` behaviour. To prevent\nemail being sent, you can block it with `Bamboo.Email.block/1`.\n\n```elixir\n# some/path/within/your/app/deny_list_interceptor.ex\ndefmodule MyApp.DenyListInterceptor do\n  @behaviour Bamboo.Interceptor\n  @deny_list [\"bar@foo.com\"]\n\n  def call(email) do\n    if email.to in @deny_list do\n      Bamboo.Email.block(email)\n    else\n      email\n    end\n  end\nend\n```","ref":"readme.html#interceptors"},{"type":"extras","title":"Using Phoenix Views and Layouts - README","doc":"Phoenix is not required to use Bamboo. But if you want to use Phoenix's views\nand layouts to render emails, see [`bamboo_phoenix`] and [`Bamboo.Phoenix`].\n\n[`bamboo_phoenix`]: https://github.com/thoughtbot/bamboo_phoenix\n[`Bamboo.Phoenix`]: https://hexdocs.pm/bamboo_phoenix/Bamboo.Phoenix.html","ref":"readme.html#using-phoenix-views-and-layouts"},{"type":"extras","title":"Viewing Sent Emails - README","doc":"Bamboo comes with a handy plug for viewing emails sent in development. Now you\ndon't have to look at the logs to get password resets, confirmation links, etc.\nJust open up the sent email viewer and click the link.\n\nSee [`Bamboo.SentEmailViewerPlug`].\n\nHere is what it looks like:\n\n![Screenshot of BambooSentEmailViewer](https://cloud.githubusercontent.com/assets/22394/14929083/bda60b76-0e29-11e6-9e11-5ec60069e825.png)","ref":"readme.html#viewing-sent-emails"},{"type":"extras","title":"Mandrill Specific Functionality (tags, merge vars, templates, etc.) - README","doc":"Mandrill offers extra features on top of regular SMTP email like tagging, merge\nvars, templates, and scheduling emails to send in the future. See\n[`Bamboo.MandrillHelper`].","ref":"readme.html#mandrill-specific-functionality-tags-merge-vars-templates-etc"},{"type":"extras","title":"SendGrid Specific Functionality (templates, substitution tags, scheduled delivery, etc.) - README","doc":"SendGrid offers extra features on top of regular SMTP email like transactional\ntemplates with substitution tags. See [`Bamboo.SendGridHelper`].","ref":"readme.html#sendgrid-specific-functionality-templates-substitution-tags-scheduled-delivery-etc"},{"type":"extras","title":"JSON support - README","doc":"Bamboo comes with JSON support out of the box via the [Jason] library. To use\nit, add `:jason` to your dependencies:\n\n```elixir\n{:jason, \"~> 1.0\"}\n```\n\nYou can customize it to use another library via the `:json_library`\nconfiguration:\n\n```elixir\nconfig :bamboo, :json_library, SomeOtherLib\n```","ref":"readme.html#json-support"},{"type":"extras","title":"Testing - README","doc":"Bamboo separates email creation and email sending. Test email creation by\nasserting against the email struct created by your functions. For example,\nassuming your welcome email accepts a user recipient, provides the correct from\naddress, and provides specific text, you might test like this:\n\n```elixir\ndefmodule MyApp.EmailTest do\n  use ExUnit.Case\n\n  test \"welcome email\" do\n    user = {\"Ralph\", \"ralph@example.com\"}\n\n    email = MyApp.Email.welcome_email(user)\n\n    assert email.to == user\n    assert email.from == \"welcome@myapp.com\"\n    assert email.html_body =~ \" Thanks for joining \"\n    assert email.text_body =~ \"Thanks for joining\"\n  end\nend\n```\n\nTest email sending in integration tests by using the [`Bamboo.TestAdapter`]\nalong with [`Bamboo.Test`]. For example, assuming during the registration\nprocess of your app an email is sent to the user welcoming them to the\napplication, you might test this feature like this:\n\n```elixir\ndefmodule MyApp.RegistrationTest do\n  use ExUnit.Case\n  use Bamboo.Test\n\n  # Remember to use the `Bamboo.TestAdapter` in your test config\n\n  test \"after registering, the user gets a welcome email\" do\n    user = new_user()\n    expected_email = MyApp.Email.welcome_email(user.email)\n\n    MyApp.Registration.create(user)\n\n    assert_delivered_email expected_email\n  end\n\n  defp new_user do\n    # Build a user appropriate to your application\n  end\nend\n```\n\nSee the documentation for [`Bamboo.Test`] for more examples and additional\nhelper functions.","ref":"readme.html#testing"},{"type":"extras","title":"Available Adapters - README","doc":"Here is a list of adapters that either ship with Bamboo or have been made\navailable by the community. Feel free to open an issue or a PR if you'd like to\nadd a new adapter to the list.\n\n- `Bamboo.LocalAdapter` - Ships with Bamboo. Stores email in memory. Great for local development.\n- `Bamboo.MailgunAdapter` - Ships with Bamboo. Thanks to [@princemaple].\n- `Bamboo.MandrillAdapter` - Ships with Bamboo.\n- `Bamboo.SendGridAdapter` - Ships with Bamboo.\n- `Bamboo.TestAdapter` - Ships with Bamboo. Use in your test environment.\n- `Bamboo.CampaignMonitorAdapter` - See [jackmarchant/bamboo_campaign_monitor](https://github.com/jackmarchant/bamboo_campaign_monitor).\n- `Bamboo.ConfigAdapter` - See [BinaryNoggin/bamboo_config_adapter](https://github.com/BinaryNoggin/bamboo_config_adapter) declare config at runtime.\n- `Bamboo.FallbackAdapter` - See [fuelen/bamboo_fallback](https://github.com/fuelen/bamboo_fallback). Allows using multiple adapters.\n- `Bamboo.GmailAdapter` - See [parkerduckworth/bamboo_gmail](https://github.com/parkerduckworth/bamboo_gmail).\n- `Bamboo.MailjetAdapter` - See [moxide/bamboo_mailjet](https://github.com/moxide/bamboo_mailjet).\n- `Bamboo.PostmarkAdapter` - See [pablo-co/bamboo_postmark](https://github.com/pablo-co/bamboo_postmark).\n- `Bamboo.SendcloudAdapter` - See [linjunpop/bamboo_sendcloud](https://github.com/linjunpop/bamboo_sendcloud).\n- `Bamboo.SesAdapter` - See [kalys/bamboo_ses](https://github.com/kalys/bamboo_ses).\n- `Bamboo.SMTPAdapter` - See [fewlinesco/bamboo_smtp](https://github.com/fewlinesco/bamboo_smtp).\n- `Bamboo.SparkPostAdapter` - See [andrewtimberlake/bamboo_sparkpost](https://github.com/andrewtimberlake/bamboo_sparkpost).","ref":"readme.html#available-adapters"},{"type":"extras","title":"Contributing - README","doc":"Before opening a pull request, please open an issue first.\n\nOnce we've decided how to move forward with a pull request:\n\n    $ git clone https://github.com/beam-community/bamboo.git\n    $ cd bamboo\n    $ mix deps.get\n    $ mix test\n    $ mix format\n\nOnce you've made your additions and `mix test` passes, go ahead and open a PR!\n\nWe run the test suite as well as formatter checks on CI. Make sure you are using\nthe Elixir version defined in the `.tool-versions` file to have consistent\nformatting with what's being run on CI.\n\n<!-- Links -->\n\n[@princemaple]: https://github.com/princemaple\n[`bamboo.adapter`]: https://hexdocs.pm/bamboo/Bamboo.Adapter.html\n[`bamboo.deliverlaterstrategy`]: https://hexdocs.pm/bamboo/Bamboo.DeliverLaterStrategy.html\n[`bamboo.email`]: https://hexdocs.pm/bamboo/Bamboo.Email.html\n[`bamboo.formatter`]: https://hexdocs.pm/bamboo/Bamboo.Formatter.html\n[`bamboo.mandrillhelper`]: https://hexdocs.pm/bamboo/Bamboo.MandrillHelper.html\n[`bamboo.phoenix`]: https://hexdocs.pm/bamboo/Bamboo.Phoenix.html\n[`bamboo.sendgridhelper`]: https://hexdocs.pm/bamboo/Bamboo.SendGridHelper.html\n[`bamboo.sentemailviewerplug`]: https://hexdocs.pm/bamboo/Bamboo.SentEmailViewerPlug.html\n[`bamboo.tasksupervisorstrategy`]: https://hexdocs.pm/bamboo/Bamboo.TaskSupervisorStrategy.html\n[`bamboo.test`]: https://hexdocs.pm/bamboo/Bamboo.Test.html\n[`bamboo.testadapter`]: https://hexdocs.pm/bamboo/Bamboo.TestAdapter.html\n[`bamboo`]: http://github.com/beam-community/bamboo\n[available-adapters]: #available-adapters\n[compose emails using pipes]: #composing-with-pipes\n[create your own adapter]: https://hexdocs.pm/bamboo/Bamboo.Adapter.html\n[docs]: https://hexdocs.pm/bamboo/readme.html\n[exq]: https://github.com/akira/exq\n[free bamboo screencasts from ElixirCasts]: https://elixircasts.io/sending-email-with-bamboo-part-1\n[jason]: https://github.com/michalmuskala/jason\n[toniq]: https://github.com/joakimk/toniq","ref":"readme.html#contributing"},{"type":"extras","title":"Upgrading to Bamboo 2.0","doc":"Upgrading to Bamboo 2.0\n=======================\n\nBamboo 2.0 ships with some breaking changes (hence the major version bump). But\ndon't worry, if you love Bamboo just as it is, there's an easy upgrade path.\n\nThere are two breaking changes:\n\n- `Bamboo.Phoenix` extracted to [bamboo_phoenix]\n- `Bamboo.Mailer.deliver_now/2` and `deliver_later/2` don't raise on errors\n\nLet's cover each in turn.","ref":"upgrade_to_2_0.html"},{"type":"extras","title":"Breaking change: `Bamboo.Phoenix` extracted to `bamboo_phoenix` - Upgrading to Bamboo 2.0","doc":"`Bamboo.Phoenix` has been extracted to the [bamboo_phoenix] library. If you use\n`Bamboo.Phoenix` to render your email templates, add `bamboo_phoenix` to your\ndependencies:\n\n```elixir\ndefp deps do\n  [\n    ...\n    {:bamboo, \"~> 2.0\"},\n    {:bamboo_phoenix, \"~> 1.0\"}\n    ...\n  ]\nend\n```","ref":"upgrade_to_2_0.html#breaking-change-bamboo-phoenix-extracted-to-bamboo_phoenix"},{"type":"extras","title":"Breaking change: `deliver_now/2`/`deliver_later/2` return `:ok` & `:error` tuples - Upgrading to Bamboo 2.0","doc":"`Bamboo.Mailer`'s `deliver_now/2` and `deliver_later/2` no longer raise errors.\nInstead, they now return an `{:ok, email}` and `{:error, error}`, where the\n`error` is an exception struct or an error message. If you pass `response:\ntrue` as an argument, the return value will be `{:ok, email, response}`.\n\nIf you prefer seeing code, this is the change in `@spec` signature for\n`deliver_now/2`:\n\n```diff\n-  @spec deliver_now(Bamboo.Email.t(), Enum.t()) :: Bamboo.Email.t() | {Bamboo.Email.t(), any}\n+  @spec deliver_now(Bamboo.Email.t(), Enum.t()) ::\n+          {:ok, Bamboo.Email.t()}\n+          | {:ok, Bamboo.Email.t(), any}\n+          | {:error, Exception.t() | String.t()}\n```\n\nNote that `deliver_later/2` will only return errors that happen _prior_ to\nscheduling the delivery of the email. What happens once the delivery is\nscheduled depends on what [delivery strategy] you are using.\n\nThose who want to handle errors on their own can now pattern match on the `:ok`\nand `:error` tuple responses. If you don't want to handle the errors and like\nhow Bamboo behaves prior to 2.0, there's a simple upgrade path. 👇","ref":"upgrade_to_2_0.html#breaking-change-deliver_now-2-deliver_later-2-return-ok-error-tuples"},{"type":"extras","title":"Simple upgrade path - Upgrading to Bamboo 2.0","doc":"`Bamboo.Mailer` comes with `deliver_now!/2` and `deliver_later!/2`. Those two\nfunctions mirror the behavior that `deliver_now/2` and `deliver_later/2` had\nbefore 2.0.\n\nHopefully, that makes for a simple upgrade path for those who don't want to\nhandle the `{:ok, email}` and `{:error, error}` tuples. You only need to\nchange:\n\n- `deliver_now/2` to `deliver_now!/2`, and\n- `deliver_later/2` to `deliver_later!/2`\n\nNote that `deliver_later!/2` will only raise email validation errors _before_\nscheduling the email delivery. What happens after the delivery is scheduled\ndepends on the [delivery strategy] you are using (e.g.\n`TaskSupervisorStrategy`).\n\n### `TaskSupervisorStrategy`\n\nRegardless of whether you use `deliver_later/2` or `deliver_later!/2`, if you\nuse the `TaskSupervisorStrategy` for delivering emails, it will continue to\nraise errors when emails fail to be delivered.\n\nIf you want control of those errors, you can implement a custom [delivery\nstrategy], to handle errors coming from `adapter.deliver`.\n\nFor now, `TaskSupervisorStrategy` continues to work as it did prior to `2.0`,\nso no change is needed here to upgrade to Bamboo 2.0.","ref":"upgrade_to_2_0.html#simple-upgrade-path"},{"type":"extras","title":"Check with your adapter - Upgrading to Bamboo 2.0","doc":"Each adapter needs to upgrade to satisfy the new [adapter.deliver callback].\nCheck with your adapter to see if it supports the new `ok` and `error` tuple\nAPI before upgrading to Bamboo 2.0.\n\nIf you use SendGrid, Mailgun, or Mandrill, your adapter is already updated with\nBamboo 2.0.\n\nThat's it! For a full list of changes, please refer to the [changelog].\n\nAnd if you find any issues with this upgrade guide, please let us know by\n[opening an issue] or submitting a pull-request.\n\n[adapter.deliver callback]: https://hexdocs.pm/bamboo/2.0.0/Bamboo.Adapter.html#c:deliver/2\n[delivery strategy]: https://hexdocs.pm/bamboo/2.0.0/Bamboo.DeliverLaterStrategy.html\n[opening an issue]: https://github.com/thoughtbot/bamboo/issues\n[bamboo_phoenix]: https://hexdocs.pm/bamboo_phoenix\n[changelog]: https://github.com/thoughtbot/bamboo/blob/master/CHANGELOG.md","ref":"upgrade_to_2_0.html#check-with-your-adapter"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}